"""Range exercises"""


def get_factors():
    """Return a list of all factors of the given number."""


def is_range():
    """Return True if given list could be represented by range with step 1."""


def annual_return():
    """Return list that models annual return for a financial asset."""


def is_prime():
    """Return True if candidate number is prime."""


def power_list():
    """Return a list that contains each number raised to the i-th power."""


def reverse_difference():
    """Return list of reversed numbers subtracted from itself."""


def with_previous():
    """Provide each list item with item before it."""


def identity():
    """Return an identity matrix of size x size."""


def matrix_add():
    """Add corresponding numbers in given 2-D matrices."""


def parse_row():
    """Return dictionary of keys from first CSV row and values from second."""


def interleave():
    """Return list of one item at a time from each given iterable."""


def triples():
    """Return list of Pythagorean triples less than input num."""
